==> lm78-1.1.2.tgz: lm78-1.1.2/lm78.c <==
/* -*- linux-c -*- */


/*
    lm78.c - A Linux module for reading sensor data.
    Copyright (c) 1997, 1998  Alexander Larsson <alla@lysator.liu.se>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


/*
 *  LM78 sensor driver for LINUX
 *
 * Copyright (c) 1997 Alexander Larsson
 * 
 * provides a /proc/sensors which shows
 * some information about your system.
 * Voltages, mb temperature and fan speeds
 * are currently reported.
 *
 * This should work on all ASUS TX97 cards
 * but it's only tested on a TX97-X.
 * It should also work on any card that has
 * a LM78 circuit, but some constants in
 * lm78.h probably has to be changed.
 *
 * please report any bugs/patches to me:
 *  alla@lysator.liu.se
 *
 * Heavily patched by Frodo Looijaard <frodol@dds.nl> and Philip 
 * Edelbrock <phil@Stimpy.netroedge.com>.
 *
 * Reference:
 *   National Semiconductor
 *   LM78 Microprocessor System Hardware Monitor, September 1996.
 *   http://www.national.com/ds/LM/LM78.pdf
 *
 */

#define LM78_VERSION  "0.01"

#include <linux/config.h>

/* Added 19980620 by Frodo Looijaard (frodol@dds.nl)
   This solves undefined symbol errors for depmod/modprobe.
   Tested for Linux 2.0.34, but I do not know whether it works for other
   kernel versions. Actually, I am not even sure this is the best way to
   solve this. */
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <linux/bios32.h>
#include <asm/io.h>
#include <linux/pci.h>
#include "lm78.h"

extern int Init_LM75(void);
extern void Cleanup_LM75(void);
extern int print_lm75_values (char *buf);


int LM75_Disabled = 0;


int init_module(void);
void cleanup_module(void);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
static int lm78_read(char *buf, char **start, off_t offset,
                     int len, int *eof, void *private);
#else
static int lm78_read(char *, char **, off_t, int, int);
#endif
static int lm78_read_value(int address);
static void read_sensor_data(void);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
MODULE_AUTHOR("Alexander Larsson <alla@lysator.liu.se>, Frodo Looijaard <frodol@dds.nl>, Philip Edelbrock <phil@Ren.netroedge.com>");
MODULE_DESCRIPTION("Driver for reading sensor values from a lm78 on the motherboard.");
#endif

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
 static struct proc_dir_entry *proc_sensors = NULL;
#else
 static int inodeval = 0;
 struct proc_dir_entry dir =
 {
   0, 7, "sensors", 
   S_IFREG | S_IRUGO, 1, 0, 0,
   0, NULL,
   &lm78_read
 };
#endif

 static struct sensor_data sensor_data = {
   0
 };


/*
 *	Initialize the module.
 */
int init_module(void)
{
  int res;
  
  if (check_region(LM78_BASE, LM78_EXTENT)) 
    return -ENODEV;

  /* Verify that lm78 exists: */
  outb_p( LM78_DUMMY, LM78_ADDRESS_REG );
  res = inb_p(LM78_ADDRESS_REG);
  inb_p(LM78_DATA_REG); /* reset busy flag */

  if ((res & 0x7F) != LM78_DUMMY) 
    return -EIO; 

  request_region(LM78_BASE, LM78_EXTENT, "LM78");

  /* Reset all except Watchdog values and last conversion values 
     This sets fan-divs to 2, among others */
  LM78_WRITE(LM78_CONFIG, 0x80 );

  /* Start monitoring */
  LM78_WRITE(LM78_CONFIG, (LM78_READ(LM78_CONFIG) & 0xf7) | 0x01);

  sensor_data.last_read=jiffies;
  sensor_data.valid=0;

  /* register /proc/sensors */
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
  
  proc_sensors = create_proc_entry("sensors", 0, 0);
  if (proc_sensors) {
    proc_sensors->read_proc = lm78_read;
  } else {
    printk("Couldn't create /proc/sensors\n");
    release_region(LM78_BASE, LM78_EXTENT);
    return -ENOENT;
  }
  
#else
  
/*
 *	Try to register the file in the root directory of the proc
 *	filesystem.  Dynamic registration means we won't provide a fixed
 *	inode number, but rather the registration code will allocate one for
 *	us.  This is safer, since we never know when someone will add a new
 *	fixed inode number to the filesystem.  Possible returns are 0
 *	(success) and -ENOENT (failure), and since there's only one error we
 *	won't worry unduly about messages.  If successful, continue.
 */
  if (proc_register_dynamic(&proc_root, &dir)) {
    release_region(LM78_BASE, LM78_EXTENT);
    return -ENOENT;
  }
  
/*
 *      Save the inode value from the registration call, as we will need
 *      it to unregister.
 */
  inodeval = dir.low_ino;

#endif
  
  printk("LM78 detected and installed.\n");
  if ((LM75_Disabled == 0) && ((LM75_Disabled = Init_LM75()) == 0))
	printk("SMBus/LM75 detected and installed.\n");
  return 0;
}

/*
 *	Remove the module.  If a nonzero inode value was returned,
 *	unregister the /proc directory entry; that's all we need to do.
 */

void cleanup_module(void)
{
        release_region(LM78_BASE, LM78_EXTENT);
        if (! LM75_Disabled)
                Cleanup_LM75();
      
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
        if (proc_sensors)
		proc_unregister(&proc_root, proc_sensors->low_ino);
#else
        if (inodeval)
		proc_unregister(&proc_root, inodeval);
#endif
}

/*
 *	Process a read to the file.  we just return the whole mess to the
 *	user with a new-line character on the end.
 */

static int lm78_read_value(int address) {
  while (LM78_BUSY) {
    current->state = TASK_INTERRUPTIBLE;
    current->timeout = jiffies + 10;
    schedule();
  }
  
  return LM78_READ(address);
}


void read_sensor_data(void)
{
  /* Note that check for the jiffies counter overflow. Note that we
     assume that when it overflows, it will be reset to 0. If not,
     we may probe once too many (too bad). */
  if ((jiffies - sensor_data.last_read < 1*HZ ) && 
      !((jiffies < sensor_data.last_read) && (jiffies < 1*HZ)))
    return; /* Less than one second ago. Not updated anyway. */

  /* TODO: grab Spin lock on sensor_data */

  sensor_data.valid = 1; 

#ifdef LM78_VCORE
  sensor_data.Vcore = ((long) lm78_read_value(LM78_VCORE)) * 16 * NORMAL_VCORE / 100000;
#endif

#ifdef LM78_3_3V
  sensor_data.V3_3v = ((long) lm78_read_value(LM78_3_3V)) * 16 * NORMAL_3_3V / 100000;
#endif

#ifdef LM78_5V
  sensor_data.V5v = ((long) lm78_read_value(LM78_5V)) * 16 * NORMAL_5V / 100000;
#endif

#ifdef LM78_M5V
  sensor_data.Vm5v = ((long) lm78_read_value(LM78_M5V)) * 16 * NORMAL_M5V / 100000;
#endif

#ifdef LM78_12V
  sensor_data.V12v = ((long) lm78_read_value(LM78_12V)) * 16 * NORMAL_12V / 100000;
#endif

#ifdef LM78_M12V
  sensor_data.Vm12v = ((long) lm78_read_value(LM78_M12V)) * 16 * NORMAL_M12V / 100000;
#endif
  
  sensor_data.temp_mb= lm78_read_value(LM78_TEMPERATURE);

  sensor_data.fan[0] = LM78_READ( LM78_FAN1 );
  sensor_data.fan[1] = LM78_READ( LM78_FAN2 );
  sensor_data.fan[2] = LM78_READ( LM78_FAN3 );

  sensor_data.last_read = jiffies;
  
  /* TODO: release Spin lock on sensor_data */
}


/*
 * Ahem, gotta find out how this is supposed to work, ie. how big is 'buf' 
 */
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
static int lm78_read(char *buf, char **start, off_t offset, int len, int *eof, void *private)
#else
static int lm78_read(char *buf, char **start, off_t offset, int len, int unused)
#endif
{
  int fan;
  long v;
  len=0;

  read_sensor_data();

  if (sensor_data.valid) {
  
#ifdef LM78_VCORE
    v = sensor_data.Vcore;
    len += sprintf(buf+len, "Vcore:  %2d.%02dV\n", (int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "Vcore: not supported\n");
#endif

#ifdef LM78_3_3V
    v = sensor_data.V3_3v;
    len += sprintf(buf+len, "+3.3V:  %2d.%02dV\n", (int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "+3.3V: not supported\n");
#endif

#ifdef LM78_5V
    v = sensor_data.V5v;
    len += sprintf(buf+len, "+5.0V:  %2d.%02dV\n", (int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "+5.0V: not supported\n" );
#endif

#ifdef LM78_M5V
    v = sensor_data.Vm5v;
    len += sprintf(buf+len, "-5.0V: %3d.%02dV\n", -(int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "-5.0V: not supported\n" );
#endif

#ifdef LM78_12V
    v = sensor_data.V12v;
    len += sprintf(buf+len, "+12V:   %2d.%02dV\n", (int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "+12V : not supported\n" );
#endif

#ifdef LM78_M12V
    v = sensor_data.Vm12v;
    len += sprintf(buf+len, "-12V:  %3d.%02dV\n", (int) -v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "-12V : not supported\n" );
#endif
  
    len += sprintf(buf+len, "Mainboard temperature: %d degrees\n",
                   sensor_data.temp_mb );

    fan = sensor_data.fan[0];
    len += sprintf(buf+len, "Chassis fan: ");
    if (fan==255)
      len += sprintf(buf+len, "0 rpm (not present?)\n");
    else
      len += sprintf(buf+len, "%d rpm\n", LM78_RPM(fan,2));


    fan = sensor_data.fan[1];
    len += sprintf(buf+len, "CPU fan: ");
    if (fan==255)
      len += sprintf(buf+len, "0 rpm (not present?)\n");
    else
      len += sprintf(buf+len, "%d rpm\n", LM78_RPM(fan,2));


    fan = sensor_data.fan[2];
    len += sprintf(buf+len, "Power fan: ");
    if (fan==255)
      len += sprintf(buf+len, "0 rpm (not present?)\n");
    else
      len += sprintf(buf+len, "%d rpm\n", LM78_RPM(fan,2));

    /* Tack on LM75 stats */
    if (! LM75_Disabled)
	len += print_lm75_values (buf+len);
  }
  else
    len += sprintf(buf+len, "No sensor data yet (try again in a few moments)\n");

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,1,0))
  *start = buf + offset;
  return len > offset ? len - offset : 0;
#else
  return len;
#endif
}

==> lm78-1.1.4.tgz: lm78-1.1.4/lm78.c <==
/*
    lm78.c - Part of a Linux module for reading sensor data.
    Copyright (c) 1997, 1998  Alexander Larsson <alla@lysator.liu.se>,
    Frodo Looijaard <frodol@dds.nl> and Philip Edelbrock <phil@netroedge.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/config.h>

/* The next define disables the redefinition of kernel_version. As it is
   already defined and initialized in lm78.c, this would result in twice
   defined symbols. We must include <linux/version.h> by hand now. */
#define __NO_VERSION__

/* Hack to allow the use of CONFIG_MODVERSIONS. In the kernel itself, this
   is arranged from the main Makefile. */
#if CONFIG_MODVERSIONS==1
#define MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/ioport.h>
#include "lm78.h"
#include "main.h"

/* These are functions used only internally. */
static int lm78_read_value(int address);
static void read_sensor_data(void);

struct sensor_data {
  unsigned long last_read;

  long Vcore;
  long V3_3v;
  long V5v;
  long Vm5v;
  long V12v;
  long Vm12v;

  int temp_mb;
  int temp_cpu;

  int fan[3];
  int valid;
};

static struct sensor_data sensor_data;


/* This function initializes the LM78. If it is found, and initializing
   succeeds, 0 is returned; if not, an error value is returned. */
int Init_LM78(void)
{
  int res;

  if (check_region(LM78_BASE, LM78_EXTENT))
    return -ENODEV;

  outb_p( LM78_DUMMY, LM78_ADDRESS_REG );
  res = inb_p(LM78_ADDRESS_REG);
  inb_p(LM78_DATA_REG); /* reset busy flag */

  if ((res & 0x7F) != LM78_DUMMY)
    return -EIO;

  request_region(LM78_BASE, LM78_EXTENT, "lm78");

  /* Reset all except Watchdog values and last conversion values
     This sets fan-divs to 2, among others */
  LM78_WRITE(LM78_CONFIG, 0x80 );

  /* Start monitoring */
  LM78_WRITE(LM78_CONFIG, (LM78_READ(LM78_CONFIG) & 0xf7) | 0x01);

  sensor_data.last_read=jiffies;
  sensor_data.valid=0;

  return 0;
}

/* This function cleans up after the module. Only call it after a succesful
   Init_LM78 call! */
void Cleanup_LM78(void)
{
   release_region(LM78_BASE, LM78_EXTENT);
}

/* This function tries to update the current sensor data and to put it
   in human-readable form in buf. It returns the number of characters
   written to buf. 
   TODO: Check length of buf. */
int print_lm78_values (char *buf)
{
  int len = 0;
  long v;
  int fan;

  read_sensor_data();
  if (sensor_data.valid) {

#ifdef LM78_VCORE
    v = sensor_data.Vcore;
    len += sprintf(buf+len, "Vcore:  %2d.%02dV\n", (int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "Vcore: not supported\n");
#endif

#ifdef LM78_3_3V
    v = sensor_data.V3_3v;
    len += sprintf(buf+len, "+3.3V:  %2d.%02dV\n", (int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "+3.3V: not supported\n");
#endif

#ifdef LM78_5V
    v = sensor_data.V5v;
    len += sprintf(buf+len, "+5.0V:  %2d.%02dV\n", (int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "+5.0V: not supported\n" );
#endif

#ifdef LM78_M5V
    v = sensor_data.Vm5v;
    len += sprintf(buf+len, "-5.0V: %3d.%02dV\n", -(int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "-5.0V: not supported\n" );
#endif

#ifdef LM78_12V
    v = sensor_data.V12v;
    len += sprintf(buf+len, "+12V:   %2d.%02dV\n", (int) v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "+12V : not supported\n" );
#endif

#ifdef LM78_M12V
    v = sensor_data.Vm12v;
    len += sprintf(buf+len, "-12V:  %3d.%02dV\n", (int) -v/100, (int) v%100 );
#else
    len += sprintf(buf+len, "-12V : not supported\n" );
#endif

    len += sprintf(buf+len, "Mainboard temperature: %d degrees\n",
                   sensor_data.temp_mb );

    fan = sensor_data.fan[0];
    len += sprintf(buf+len, "Chassis fan: ");
    if (fan==255)
      len += sprintf(buf+len, "0 rpm (not present?)\n");
    else
      len += sprintf(buf+len, "%d rpm\n", LM78_RPM(fan,2));


    fan = sensor_data.fan[1];
    len += sprintf(buf+len, "CPU fan: ");
    if (fan==255)
      len += sprintf(buf+len, "0 rpm (not present?)\n");
    else
      len += sprintf(buf+len, "%d rpm\n", LM78_RPM(fan,2));

    fan = sensor_data.fan[2];
    len += sprintf(buf+len, "Power fan: ");
    if (fan==255)
      len += sprintf(buf+len, "0 rpm (not present?)\n");
    else
      len += sprintf(buf+len, "%d rpm\n", LM78_RPM(fan,2));
  }
  else
    len += sprintf(buf+len, "No sensor data yet (try again in a few moments)\n");
  return len;
}


/* Read a value from the LM78. Theoretically, several drivers can use
   the LM78 at once by using this function. In practice, this is unlikely,
   but still it won't do any harm. */
int lm78_read_value(int address) 
{
  while (LM78_BUSY) {
    current->state = TASK_INTERRUPTIBLE;
    current->timeout = jiffies + 10;
    schedule();
  }
  return LM78_READ(address);
}

/* Reread the sensor information into struct sensor_data. */
void read_sensor_data(void)
{
  /* Note that check for the jiffies counter overflow. Note that we
     assume that when it overflows, it will be reset to 0. If not,
     we may probe once too many (too bad). */
  if ((jiffies - sensor_data.last_read < 1*HZ ) &&
      !((jiffies < sensor_data.last_read) && (jiffies < 1*HZ)))
    return; /* Less than one second ago. Not updated anyway. */

  /* TODO: grab Spin lock on sensor_data */

  sensor_data.valid = 1;

#ifdef LM78_VCORE
  sensor_data.Vcore = ((long) lm78_read_value(LM78_VCORE)) * 16 * NORMAL_VCORE / 100000;
#endif

#ifdef LM78_3_3V
  sensor_data.V3_3v = ((long) lm78_read_value(LM78_3_3V)) * 16 * NORMAL_3_3V / 100000;
#endif

#ifdef LM78_5V
  sensor_data.V5v = ((long) lm78_read_value(LM78_5V)) * 16 * NORMAL_5V / 100000;
#endif

#ifdef LM78_M5V
  sensor_data.Vm5v = ((long) lm78_read_value(LM78_M5V)) * 16 * NORMAL_M5V / 100000;
#endif

#ifdef LM78_12V
  sensor_data.V12v = ((long) lm78_read_value(LM78_12V)) * 16 * NORMAL_12V / 100000;
#endif

#ifdef LM78_M12V
  sensor_data.Vm12v = ((long) lm78_read_value(LM78_M12V)) * 16 * NORMAL_M12V / 100000;
#endif

  sensor_data.temp_mb= lm78_read_value(LM78_TEMPERATURE);

  sensor_data.fan[0] = lm78_read_value( LM78_FAN1 );
  sensor_data.fan[1] = lm78_read_value( LM78_FAN2 );
  sensor_data.fan[2] = lm78_read_value( LM78_FAN3 );

  sensor_data.last_read = jiffies;

  /* TODO: release Spin lock on sensor_data */
}
